/*								7/2/98 by Simon van Norden
					   send comments & suggestions to svn@alum.mit.edu
=======================================================================================
This program references a program by David Baird that calculates critical values for the Kolmogorov-Smirnov statistics. His program (extracted from a larger body of work) and documentation are appended at the end of this file. Note that he requests you reference his contribution in any published work, and that his code is provided as freeware for public non-commercial use.
=======================================================================================

PVALPLOT()

This program is designed for use with simulated data. Given a matrix of data x, it generates the data's theoretical p-values, G(x) (based on a user-provided distribution.) It then constructs p-values based on the empirical CDF for the data, F(x). Finally, it then generates two sets of graphs. The first is a "P-Value Plot", which simply plots F(x) vs G(x).  If the theoretical distribution is correct, we will see a 45-degree line. The second is a "P-Value Discrepency Plot", which plots F(x) - G(x) vs G(x).  If the theoretical distribution is correct, we will see a horizontal line at 1. A 95% confidence interval for the Kolmogorov-Smirnov statistic is shaded; if the discrepency plot goes outside this range, we can reject the null that the empirical distribution is the same as the theoretical distribution.

Each type of graph is plotted for the full range of p-values (0 to 1.0), and then for the range (0 to 0.1) most used for inference. The reference for these graphs is Davidson and MacKinnon (1997). 

The syntax of the program is 
	xy = pvalplot(x,&cdfproc,dof);
where
	x		is a matrix containing the statistics you want to examine 
			(each row is one trial, each column is one statistic.)
	&cdfproc	is a pointer to a procedure which calculates the *complement* of the cdf 
			(more on this below)
	dof		is a matrix containing the degrees of freedom parameters required for 
			&cdfproc
	xy		is an N x 215 matrix containing the data points graphed in the p-value plot (column 1 is the X co-ordinates, the rest are Y co-ordinates.)

To use this procedure, you'll need a procedure to return the theoretical CDF. This procedure will be called exactly once as
	 dta = cdfproc(dta,dof);
Example of procs for the standard normal, the Chi-Squared and the F distribution are included after the body of the main procedure. If you need other distributions, it is up to you to provide a procedure.  Make sure that 
	a)	it can evaluate all the columns and rows of dta simultaneously
	b)	dof contains all the parameters which the cdfproc() might need 
This procedure also references my shadebox() proc, which shades regions on 2-D GAUSS graphs.


Before calling pvalplot, the user may want to use _plegctl and _plegstr to set the legend for the graphs, or set _pframe = {0, 1}. The user may control all aspects of the graphics, aside from the following (which are set by pvalplot()):
	ylabel, xlabel, window, scale, _ptitle.
The user may also set the global variable _pvprint; this is a string variable which is passed to graphprt() after constructing the image.
*/

proc pvalpreplot(dta,cdfproc,dof);

local cdfproc:proc, nobs, ncol, i, pval, xj;

@set up some constants@
nobs=rows(dta);
ncol=cols(dta);
pval=zeros(215,ncol);
xj = seqa(.001,.001,10)  | seqa(.015, .005,195) | seqa(.99, .001, 10);


@calculate p-values using theoretical distribution@
dta = cdfproc(dta,dof);

@calculate empirical CDF of p-values@
for i (1,ncol,1);
	pval[.,i] = counts(dta[.,i],xj);
endfor;
pval = cumsumc(pval)/nobs;

retp(xj~pval);
endp;




proc redopvalplot(xydta,nobs);

local ncol, i, pval, xj, ks_crit, boxdimen;

@set up some constants@
ncol=cols(xydta);
pval=xydta[.,2:ncol];
xj = xydta[.,1];

@Now do the graphics@

/* setup globals for all graphs */
xlabel("Theoretical p-value");
_plctrl = 0;
_pstype = 4;
_psymsiz = 1;

/* create 4 windows */
begwind;
window(2,2,0);

setwind(1);
_ptitle = "P-Value Plot";
scale(0|1, 0| 1);
ylabel("Empirical p-value");
xy(xj,xj~pval);

nextwind;
scale(0|0.1, 0| miss(1,1));
xy(xj[1:28,.],xj[1:28,.]~pval[1:28,.]);

_ptitle = "P Value Discrepancy Plot";
ylabel("Empirical - Theoretical p-value");

nextwind;
ks_crit = cv_ks(0.025,nobs);
scale(0|1, miss(1,1)| miss(1,1));
boxdimen = 0~ks_crit | 1~-ks_crit;
_pline = shadebox(boxdimen,0);
xy(xj,zeros(215,1)~(pval - xj));


nextwind;
scale(0|0.1, miss(1,1)| miss(1,1));
boxdimen = 0~ks_crit | 0.1~-ks_crit;
_pline = shadebox(boxdimen,0);
xy(xj[1:28,.],zeros(28,1)~(pval[1:28,.] - xj[1:28,.]));
endwind;

_pline = 0;

retp(xj~pval);
endp;



proc pvalplot(dta,cdfproc,dof);

local cdfproc:proc, nobs, pval;

pval = pvalpreplot(dta,&cdfproc,dof);
nobs=rows(dta);
retp(redopvalplot(pval,nobs));

endp;


proc pval_chi2(dta,dof);
	retp(cdfchic(dta,dof));
endp;

proc pval_normal(dta,dof);
	retp(cdfnc(dta));
endp;

proc pval_f(dta,dof);
	retp(cdffc(dta,dof[.,1],dof[.,2]));
endp;


@ The following proc for the Kolmogorov-Smirnov critical values was excerpted from a library of goodness of fit tests by David Baird. @

/**************** Goodness of Fit ************************
This library of goodness of fit statistics was
written by David Baird and may be distributed as freeware
for public non-commercial use. Please provide appropriate
acknowledgment if this library supports published work.
There are no performance guarantees for this code.
Last update: 11/9/94

  Dr David Baird  Biometrician
  Mail:           AgResearch, PO Box 60, Lincoln, NEW ZEALAND
  Phone:          +64 3 325 6900    Fax: +64 3 325 2946
  Direct Dial:    +64 3 325 6901 3975#
  Internet:       BairdD@AgResearch.CRI.NZ
***********************************************************/

/* CV_KS - Calculate critical value for Kolmogorov-Smirnov Statistic
**
** The KS statistic is a goodness of fit statistic to test whether two
** cumulative frequency distributions are significantly different.
** The KS statistic is the maximum difference between the CDF's of the
** two distrubutions
**
** Input:  P - R x C matrix of probabilities ( 0 < p < 1 )
**         N - S x T matrix of distribution size, ExE conformable with P
**
** Output: C - R x C matrix of critical values ie Prob(AD > C) = P
**
** Usage   C = CV_KS(0.05,80) ;
**
** Notes:  C is only returned for N > 5 and P in (0.0001,0.2)
**         C is invariant to the value of N
**         PR_KS provides the inverse tranform, and estimates P given C,N
*/

proc cv_ks(p,n) ;
     local c0,c1,c2 ;
     if not (0.0001 <= p and p <= 0.2) ;
        errorlog "ERROR: CV_KS - p value out of range (0.0001,0.2)" ;
        retp(error(1)) ;
     elseif not (n > 4) ;
        errorlog  "ERROR: CV_KS - n value too small (n <= 4)" ;
        retp(error(1)) ;
     endif ;
     p = ln(p) ;
     n = 1./sqrt(n) ;
     c0 =  0.733183379676 ;
     c1 = -0.229138856954 - n.*0.016445061725 ;
     c2 = -0.007245795230 - n.*0.005111922399 ;
     retp((c0 + p.*(c1 + p.*c2))./(1./n + 0.11 + 0.12.*n)) ;
endp ;
